unit LTB_Client_ChartForm;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ExtCtrls, Vcl.StdCtrls,
  VCLTee.Chart, VCLTee.Series, VCLTee.TeeTools, VCLTee.TeeProcs, VCLTee.TeEngine;

type
  TChartThread = class(TThread)
  private
    relevantEvents: array[0..1] of THandle;
  public
    constructor create;
    procedure Execute; override;
  end;

  TForm1 = class(TForm)
    pnlChart: TPanel;
    pnlControl: TPanel;
    btnStart: TButton;
    lblFramesCounter: TLabel;
    tmrFramescounter: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormDestroy(Sender: TObject);
    procedure btnStartClick(Sender: TObject);
    procedure tmrFramescounterTimer(Sender: TObject);
  private
    fChartThread: TChartThread;
    fStart: Boolean;
    fCamFramescounter : Integer;
    fLastgettickcount: Cardinal;
    fChart: TChart;
    fChartTool1: TAxisArrowTool;
    fChartTool2: TAxisArrowTool;
    fTerminationEvent: THandle;
    fViewThreadTerminationEvent: THandle;
    fArrayX : Array of double;
    fArrayY : Array of double;
    procedure NewExposure;
    procedure RefreshChart;
    procedure BlockChart(aOnOff: boolean);
    procedure PlotMeasurement;
    procedure DeleteCurrentSpektrumSeries;
    procedure BringComparisonCurvesToFront;
  end;

var
  Form1: TForm1;

implementation


{$R *.dfm}

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  SetEvent(fTerminationEvent);
  if Assigned(fChartThread) then
  begin
    fChartThread.WaitFor;
    FreeAndNil(fChartThread);
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  fStart:= False;
  fTerminationEvent := CreateEvent(nil, true, false, nil);
  fViewThreadTerminationEvent := CreateEvent(nil,True,False, nil);

  fChartThread := TChartThread.create;

  fChart:= TChart.Create(self);
  fChart.Parent := self;
  fChart.Left := 0;
  fChart.Top := 0;
  fChart.Width := 700;
  fChart.Height := 500;
  fChart.MarginBottom := 1;
  fChart.MarginLeft := 1;
  fChart.MarginRight := 2;
  fChart.MarginTop := 2;
  fChart.PrintProportional := False;
  fChart.Title.Text.Clear;
  fChart.Title.AdjustFrame := False;
  fChart.Title.Font.Size := 12;
  fChart.Title.Font.Style := [fsItalic, fsBold];
  fChart.Title.Alignment := taLeftJustify;

  fChart.Zoom.Allow := true;
  fChart.Zoom.Animated := True;
  fChart.Zoom.Pen.Visible := true;
  fChart.Zoom.Pen.Color := clBlack;
  fChart.Zoom.Pen.Style := psDot;

  fChart.BackImageInside := true;
  fChart.BackImageMode := pbmCenter;
  fChart.BackImageTransp := true;

  fChart.Walls.Back.Pen.Style := psClear;
  fChart.View3DWalls := true;

  fChart.Chart3DPercent := 1;
  fChart.View3D := False;
  fChart.Align := alClient;
  fChart.BevelOuter := bvNone;
  fChart.TabOrder := 0;
  fChart.AutoRepaint := false;

  fChart.Foot.Text.Clear;
  fChart.Foot.Text.Add('');
  fChart.Foot.AdjustFrame := true;
  fChart.Foot.Alignment := taLeftJustify;
  fChart.Foot.Font.size := 8;
  fChart.Foot.Font.Color := clBlack;
  fChart.Foot.Font.Style := [];

  fChart.BottomAxis.AxisValuesFormat := '#,##0.######';
  fChart.BottomAxis.TickOnLabelsOnly := False;
  fChart.BottomAxis.Axis.Visible := false;
  fChart.BottomAxis.ExactDateTime := False;
  fChart.BottomAxis.MinorTickCount := 4;
  fChart.BottomAxis.MinorTickLength := 5;
  fChart.BottomAxis.TickLength := 5;
  fChart.BottomAxis.Ticks.Color := clRed;
  fChart.BottomAxis.Ticks.Style := psDash;
  fChart.BottomAxis.TickOnLabelsOnly := False;
  fChart.BottomAxis.MinorTicks.Style := psDash;

  fChart.LeftAxis.Automatic := true;
  fChart.LeftAxis.AxisValuesFormat := '#,##0.######';
  fChart.LeftAxis.MinorTickCount := 4;
  fChart.LeftAxis.MinorTickLength := 5;
  fChart.LeftAxis.TickLength := 5;
  fChart.LeftAxis.TickOnLabelsOnly := false;
  fChart.LeftAxis.Axis.Visible := false;
  fChart.LeftAxis.Ticks.Color := clRed;
  fChart.LeftAxis.Logarithmic := false;

  fChart.RightAxis.Axis.Visible := false;
  fChart.TopAxis.Axis.Visible := false;

  fChart.Legend.Alignment := laBottom;
  fChart.Legend.CheckBoxes := True;
  fChart.Legend.ColorWidth := 0;
  fChart.Legend.DividingLines.Color := clRed;
  fChart.Legend.DividingLines.Style := psDash;
  fChart.Legend.DividingLines.Width := 3;
  fChart.Legend.FontSeriesColor := True;
  fChart.Legend.LegendStyle := lsSeries;
  fChart.Legend.Symbol.Width := 0;
  fChart.Legend.TextStyle := ltsRightValue;
  fChart.Legend.TopPos := 5;
  fChart.Legend.VertMargin := 1;

  fChartTool1 := TAxisArrowTool.Create(Self);
  fChartTool1.Brush.Color := clRed;
  fChartTool1.Length := 25;
  fChartTool1.Axis := fChart.LeftAxis;

  fChartTool2 := TAxisArrowTool.Create(Self);
  fChartTool2.Brush.Color := clRed;
  fChartTool2.Length := 25;
  fChartTool2.Axis := fChart.BottomAxis;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  CloseHandle(fTerminationEvent);
end;

procedure TForm1.NewExposure;
var
  i : integer;
begin
  SetLength(fArrayX, 1000);
  SetLength(fArrayY, 1000);

  for i := 0 to 1000-1 do
  begin
    fArrayX[i]:= i;
    fArrayY[i]:= Random(1000);
  end;
end;

procedure TForm1.RefreshChart;
begin
  NewExposure;

  BlockChart(true);

  Inc(fCamFramescounter);

  PlotMeasurement;

  BlockChart(false);

  if fStart then
  begin
    SetEvent(fViewThreadTerminationEvent);
  end;
end;

procedure TForm1.tmrFramescounterTimer(Sender: TObject);
var
  seconds: double;
  msSinceLastCall: Cardinal;
begin
  lblFramesCounter.Visible := True;
  msSinceLastCall := GetTickCount - fLastgettickcount;
  fLastgettickcount := GetTickCount;
  seconds := msSinceLastCall/1000;
  lblFramesCounter.Caption := FloatTostrF(fCamFramescounter/seconds, ffFixed, 3,2) + ' fps';
  fCamFramescounter := 0;
end;

procedure TForm1.BlockChart(aOnOff: boolean);
begin
  if aOnOff then
  begin
    fChart.AutoRepaint := False;
  end
  else
  begin
    fChart.AutoRepaint := True;
    fChart.Repaint;
  end;
end;

procedure TForm1.PlotMeasurement;
var
  CurrentOrderSeries: TFastLineSeries;
begin
  DeleteCurrentSpektrumSeries;

  CurrentOrderSeries := TFastLineseries.create(fChart);
  try
    CurrentOrderSeries.Pen.width := 1;
    CurrentOrderSeries.Cursor := crCross;
    CurrentOrderSeries.DrawAllPoints := true;
    CurrentOrderSeries.DrawAllPointsStyle := daMinMax;
    CurrentOrderSeries.Title := 'order ' + IntToStr(1);
    CurrentOrderSeries.ShowInLegend := False;
    CurrentOrderSeries.XValues.Value := TChartValues(fArrayX);
    CurrentOrderSeries.XValues.Count := Length(fArrayX);
    CurrentOrderSeries.XValues.Modified := true;
    CurrentOrderSeries.YValues.Value := TChartValues(fArrayY);
    CurrentOrderSeries.YValues.Count := Length(fArrayY);
    CurrentOrderSeries.YValues.Modified := True;
  finally
    fChart.AddSeries(CurrentOrderSeries);
  end;

  BringComparisonCurvesToFront;
end;

procedure TForm1.BringComparisonCurvesToFront;
var
  i: integer;
begin
  i := 0;
  while i < fChart.SeriesList.Count do
  begin
    fChart.SeriesDown(fChart.SeriesList.Items[i]);
    Inc(i);
  end;
end;

procedure TForm1.DeleteCurrentSpektrumSeries;
var
  I: Integer;
  CurrSeries: TChartSeries;
begin
  I := fChart.SeriesList.Count-1;
  while I >= 0 do
  begin
    CurrSeries := fChart.SeriesList.Items[I];
    FreeAndNil(CurrSeries);
    dec(I);
  end;
end;

procedure TForm1.btnStartClick(Sender: TObject);
begin
  fStart:= not fStart;

  if fStart then
    SetEvent(fViewThreadTerminationEvent)
  else
    ResetEvent(fViewThreadTerminationEvent);
end;

constructor TChartThread.create;
begin
  inherited create(false);
  relevantEvents[0] := Form1.fTerminationEvent;
  relevantEvents[1] := Form1.fViewThreadTerminationEvent;
  SetThreadPriority(self.Handle, THREAD_PRIORITY_BELOW_NORMAL);
  freeOnTerminate := false;
end;

procedure TChartThread.Execute;
var
  waitResult : DWORD;
begin
  inherited;
  while not terminated do
  begin
    waitResult := WaitForMultipleObjects(Length(relevantEvents),@relevantEvents,false,INFINITE);
    case waitResult of
      WAIT_OBJECT_0: Terminate;
      WAIT_OBJECT_0+1:
      begin
        if assigned(Form1) then
        begin
          synchronize(Form1.RefreshChart);
        end;
      end;
    end;
  end;
end;

end.
